/**
 *    '$RCSfile: HsqlDataQueryAction.java,v $'
 *
 *     '$Author: tao $'
 *       '$Date: 2004/09/14 18:44:46 $'
 *   '$Revision: 1.2 $'
 *
 *  For Details: http://kepler.ecoinformatics.org
 *
 * Copyright (c) 2004 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

package org.ecoinformatics.seek.dataquery;

import org.ecoinformatics.util.DebugPrinter;
import org.ecoinformatics.util.Util;

import java.awt.event.ActionEvent;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;

/**
 * This class will handle data query action base on Hsql sql engine
 * @author Jing Tao
 *
 */
public class HsqlDataQueryAction extends DataQueryAction
{
  private Connection   conn                   = null;
  private String       sql                    = null;
  private DebugPrinter dbg                    = null;
  private ResultSet    rs                     = null;


  /**
   * Default constructor. In this conctructor, it will set up connection to db
   */
  public HsqlDataQueryAction() throws SQLException, ClassNotFoundException
  {
    dbg = DebugPrinter.getInstance();
    conn = Util.getDBConnection();

  }

  /**
   * Get sql command
   * @return String
   */
  public String getSQL()
  {
    return sql;
  }


  /**
   * Set sql command
   * @param mysql String
   */
  public void setSQL(String mysql)
  {
    sql = mysql;
  }
  /**
   * Method to get result set
   * @return ResultSet
   */
  public ResultSet getResultSet()
  {
    return rs;
  }

  /**
   * This method will handle such staff:
   * Load data into db(create text tables and set source file location to table)
   * Run sql query against the db
   * @param event ActionEvent
   */
  public void actionPerformed(ActionEvent event)
  {
    if (sql == null)
    {
      dbg.print("Sql query is null and user should specify it", 1);
      return;
    }
    Statement st = null;
    try
    {
      st = conn.createStatement();       // statement objects can be reused with
                                         // repeated calls to execute
    }
    catch (SQLException sqle)
    {
      dbg.print("Couldn't create a statement", 1);
      return;
    }
     // Run the sql query and get resultset
     try
    {
      rs = st.executeQuery(sql);
      //handleResultSet(rs);
    }
    catch (SQLException e)
    {
      dbg.print("The error to run sql command " + sql + " is " +
                e.getMessage(), 2);
      return;
    }
    try
    {
      st.close();
      conn.close();
    }
    catch(SQLException e)
    {
      dbg.print("There is an error to close statement " + e.getMessage(), 2);
      return;

    }

  }//actionPerformed


  /*
   * This method will transfer resultset to some output
   * We need to discuss it
   */
  private void handleResultSet(ResultSet result) throws SQLException
  {
    if (result == null)
    {
      dbg.print("result is null", 2);
      return;
    }
    ResultSetMetaData meta   = result.getMetaData();
    int               colmax = meta.getColumnCount();
    Object            obj    = null;

    // the result set is a cursor into the data.  You can only
    // point to one row at a time
    // assume we are pointing to BEFORE the first row
    // rs.next() points to next row and returns true
    // or false if there is no next row, which breaks the loop
    while (result.next())
    {
      for (int i = 0; i < colmax; i++)
      {
         obj = result.getObject(i + 1);    // Is SQL the first column is indexed
                                           // with 1 not 0
         System.out.print(obj.toString() + " ");
      }//for
      System.out.println(" ");
   }//while

  }//handleResultSet
}
