/**
 *    '$RCSfile: EcogridDataCacheItem.java,v $'
 *
 *     '$Author: rspears $'
 *       '$Date: 2004/10/07 19:09:23 $'
 *   '$Revision: 1.4 $'
 *
 *  For Details: http://kepler.ecoinformatics.org
 *
 * Copyright (c) 2003 The Regents of the University of California.
 * All rights reserved.
 * 
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 * 
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */
package org.ecoinformatics.seek.datasource;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.net.URLConnection;

import org.ecoinformatics.ecogrid.client.EcogridQueryClient;
import org.ecoinformatics.seek.querybuilder.DBUIUtils;
import org.w3c.dom.Node;

/**
 * This class is not as generic as the name may indicate. It is designed to get the metadata to determine 
 * where the data is stored and then does a Ecogrid "get" to the data. 
 */
public class EcogridDataCacheItem extends DataCacheItem
{
    protected String mEndPoint = null;
    
    /**
     * 
     *
     */
    public EcogridDataCacheItem()
    {
        super();
    }
    
    /**
     * @param aName
     * @param aResName
     * @param aSize
     */
/*    public EcogridDataCacheItem(String aName, String aResName)
    {
        super(aName, aResName);
    }*/
    
    /**
     * 
     */
    protected String getSerialClassName()
    {
      return EcogridDataCacheItem.class.getName();
    }
    
    /* (non-Javadoc)
     * @see org.ecoinformatics.seek.datasource.DataCacheItem#doWork()
     */
    public void doWork()
    {
        dbg.print("EcogridDataCacheItem - doing Work mStatus "+mStatus, 2);
        
        if (mData == null)
        {
            loadData();
        }

        if (!isReady() || mData == null)
        {
            dbg.print("EcogridDataCacheItem URL is: " + mResourceName, 2);
            
            StringBuffer dataSB = new StringBuffer();
            if (mResourceName.startsWith("http://") || mResourceName.startsWith("file://")) {
                // get the data from a URL
                try {
                    URL url = new URL(mResourceName);
                    if (url != null)
                    {
                        URLConnection conn = url.openConnection();
                        if (conn != null) 
                        {
                            InputStream filestream = url.openStream();
                            if (filestream != null)
                            {
                                InputStreamReader isr = new InputStreamReader(filestream);
                                if (isr != null) 
                                {
                                    String type = conn.getContentType();
                                    
                                    char[] c = new char[1024];
                                    int bread = isr.read(c, 0, 1024);
                                    while (bread != -1) {
                                        dataSB.append(c, 0, bread);
                                        bread = isr.read(c, 0, 1024);
                                    }
                                    dbg.print("HTTP Data is: \n" + dataSB.toString(), 2);
                                    if (dataSB.length() > 0)
                                    {
                                        mData = new byte[dataSB.length()];
                                        mSize = mData.length;
                                        for (int i=0;i<dataSB.length();i++)
                                        {
                                            mData[i] = (byte)dataSB.charAt(i);
                                        }
                                    }
                                    mStatus = CACHE_COMPLETE;
                                    saveData();
                                    return;
                                }
                            }
                        }
                    }
                    mStatus = CACHE_ERROR;
                    dbg.print("EcogridDataCacheItem - error connecting to http/file ", 2);
                } catch (IOException ioe) {
                    mStatus = CACHE_ERROR;
                }
                
            } else if (mResourceName.startsWith("ecogrid://")) {
                try {
                    // create a ecogrid client object and get the full record from the
                    // client
                    System.out.println("mEndPoint "+mEndPoint);
                    EcogridQueryClient ecogridClient = new EcogridQueryClient(new URL(mEndPoint));
                    if (ecogridClient != null)
                    {
                        ecogridClient.createEcoGridQueryLevelOnePortType();
                        int start = mResourceName.indexOf("/",11) + 1;
                        dbg.print("start: " + start, 2);
                        int end = mResourceName.indexOf("/", start);
                        if (end == -1) {
                            end = mResourceName.length();
                        }
                        dbg.print("end: " + end, 2);
                        String identifier = mResourceName.substring(start, end);
                        dbg.print("Get from EcoGrid: " + identifier, 2);
                        mData = ecogridClient.get(identifier);
                        if (mData != null) 
                        {
                            mSize = mData.length;
                            dbg.print("Bytes returned: " + mData.length, 2);
                            mStatus = CACHE_COMPLETE;
                        } else 
                        {
                            mStatus = CACHE_ERROR;
                            dbg.print("EcogridDataCacheItem - Ecogrid returned null data", 2);
                        }
                    } else 
                    {
                        mStatus = CACHE_ERROR;
                        dbg.print("EcogridDataCacheItem - error connecting to Ecogrid ", 2);
                    }
                    
                } catch (Exception ee) {
                    String error = ee.getMessage();
                    dbg.print(error, 2);
                }
            } else {
                mStatus = CACHE_ERROR;
            }
        }        
    }
    
    /**
     * 
     * @return
     */
    protected String getAttrsForXML()
    {
        return "   endpoint=\"" + mEndPoint +"\"\n";

    }
    
    /**
     * 
     * @param aNode
     */
    public boolean buildFromDOM(Node aNode)
    {
        if (aNode != null) 
        {
            mEndPoint = DBUIUtils.findAttrValue(aNode, "endpoint");
        }
        return super.buildFromDOM(aNode);
    }
    /**
     * @return Returns the mEndPoint.
     */
    public String getEndPoint()
    {
        return mEndPoint;
    }
    /**
     * @param endPoint The mEndPoint to set.
     */
    public void setEndPoint(String endPoint)
    {
        mEndPoint = endPoint;
    }
}
