/**

 *    '$RCSfile: QueryAction.java,v $'

 *

 *     '$Author: rspears $'

 *       '$Date: 2004/10/13 20:23:40 $'

 *   '$Revision: 1.5 $'

 *

 *  For Details: http://kepler.ecoinformatics.org

 *

 * Copyright (c) 2004 The Regents of the University of California.

 * All rights reserved.

 *

 * Permission is hereby granted, without written agreement and without

 * license or royalty fees, to use, copy, modify, and distribute this

 * software and its documentation for any purpose, provided that the

 * above copyright notice and the following two paragraphs appear in

 * all copies of this software.

 *

 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY

 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES

 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN

 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY

 * OF SUCH DAMAGE.

 *

 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,

 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF

 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE

 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY

 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,

 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.

 */



package org.ecoinformatics.seek.ecogrid;



import java.awt.event.ActionEvent;



import javax.swing.AbstractAction;



import org.ecoinformatics.ecogrid.QueryType;

import org.ecoinformatics.ecogrid.ResultsetType;

import org.ecoinformatics.ecogrid.client.EcogridQueryClient;

import org.ecoinformatics.seek.datasource.DataCacheItem;

import org.ecoinformatics.seek.datasource.DataCacheListener;

import org.ecoinformatics.seek.datasource.DataCacheManager;

import org.ecoinformatics.seek.datasource.EcogridQueryDataCacheItem;

import org.ecoinformatics.util.DebugPrinter;

/**

 * This class will access ecogrid nodes: send the query and get the resultset

 * @author Tao

 */



public class QueryAction extends AbstractAction implements DataCacheListener

{

  private QueryType                  query = null;

  private String                       url = null;

  private ResultsetType            results = null;

  private String               searchValue = null;

  private EcogridQueryClient ecogridClient = null;

  private DebugPrinter dbg                 = null;

  private String error                     = null;

  

  private DataCacheListener              _cacheListener     = null;

  private MetadataSpecificationInterface _metadataSpecClass = null;

  private QuickSearchAction              _searchAction      = null;

  private EcogridQueryDataCacheItem      _dataCacheItem     = null;



  /**

   * Contructor of QueryAction

   * @param myQuery QueryType

   * @param url Vector

   */

  public QueryAction(QuickSearchAction mySearchAction,

                     QueryType         myQuery, 

                     String            myUrl, 

                     String            mySearchValue, 

                     DataCacheListener myListener,

                     MetadataSpecificationInterface myMetadataSpecClass)

  {

    _searchAction      = mySearchAction;

    query              = myQuery;

    url                = myUrl;

    dbg                = DebugPrinter.getInstance();

    searchValue        = mySearchValue;

    _cacheListener     = myListener;

    _metadataSpecClass = myMetadataSpecClass;

  }



  /**

   * 

   * @return

   */

  public MetadataSpecificationInterface getMetaDataIFace()

  {

      return _metadataSpecClass;

  }

  

  /**

   * 

   * @return

   */

  public String getURL()

  {

      return url;

  }

  

 /**

  * The todo Implementation of abstract method. It will search ecogrid site

  *

  * @param e ActionEvent

  */

 public void actionPerformed(ActionEvent e)

 {

   if( query == null || url == null)

   {

     error = "Query is null or search url is null";

     dbg.print(error, 2);

     return;

   }

   try

   {

        _cacheListener.complete(null);

        DataCacheManager mgr = DataCacheManager.getInstance();

        String dataCacheName = "Ecogrid Query Search for " + searchValue;

        _dataCacheItem = (EcogridQueryDataCacheItem)mgr.getCacheItem(this, dataCacheName, url, EcogridQueryDataCacheItem.class.getName());

        _dataCacheItem.setQuery(query);

        _dataCacheItem.addListener(_cacheListener);

        _dataCacheItem.start();

   }

   catch(Exception ee)

   {

     error = ee.getMessage();

     dbg.print(error, 2);

   }

 }//actionPerformed



 /**

  * Method to get the resultset after query

  * @return ResultsetType

  */

 public ResultsetType getResultSet()

 {

   return this.results;

 }//getResultSet



 /**

  * Method to return the error message when excute query action

  * @return String

  */

 public String getError()

 {

   return error;

 }

 

 //------------------------------------------------------------------------

 //-- DataCacheListener

 //------------------------------------------------------------------------

 

 public void complete(DataCacheItem aItem)

 {

     System.out.println("complete: "+this);

     

     aItem.removeListener(this);

     

     if (aItem.isReady())

     {

         if (_dataCacheItem.isReady()) 

         {

             DataCacheManager.getInstance().saveCache();

             results = _dataCacheItem.getResultset();

             // if can't get a result, i.e, results equals null, we will conisder

             // there is some error happend.

             if (results == null)

             {

               error = "Couldn't find any results after querying";

             }

         } else

         {

             DataCacheManager.getInstance().removeItem(_dataCacheItem);

         }

         _searchAction.resultsetComplete(this);

     } else

     {

         results = null;

         _searchAction.resultsetComplete(this);

         error = "Couldn't find any results after querying";

     }

 }

}//QueryAction

