/**
 *  '$RCSfile: ResultTreeRebuilder.java,v $'
 *  Copyright: 2004 Regents of the University of California and the
 *             National Center for Ecological Analysis and Synthesis
 *
 *   '$Author: berkley $'
 *     '$Date: 2004/10/15 17:50:37 $'
 * '$Revision: 1.18 $'
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ecoinformatics.seek.ecogrid;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.JLabel;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeModel;

import java.awt.Dimension;
import java.awt.BorderLayout;

import ptolemy.kernel.util.NamedObj;
import ptolemy.kernel.util.IllegalActionException;
import ptolemy.kernel.util.NameDuplicationException;
import ptolemy.vergil.tree.PTree;
import ptolemy.kernel.CompositeEntity;
import ptolemy.kernel.ComponentEntity;
import ptolemy.kernel.Entity;
import ptolemy.kernel.util.ChangeRequest;
import ptolemy.moml.MoMLChangeRequest;
import ptolemy.vergil.tree.VisibleTreeModel;
import ptolemy.kernel.util.Workspace;
import ptolemy.moml.EntityLibrary;
import ptolemy.actor.CompositeActor;
import ptolemy.kernel.util.Attribute;

import java.util.Iterator;
import java.util.Vector;
import java.io.StringWriter;
import java.io.IOException;


/**
 * This class builds the search results by traversing the tree and trimming
 * any sub-nodes that do not have a result in them.  This leaves a minimum
 * tree with only the search results present.
 *
 * @author Chad Berkley
 */
public class ResultTreeRebuilder extends LibrarySearchResultPane
{
  private VisibleTreeModel trimmedLibrary;
  private EntityLibrary newRoot;
  private PTree resultsTree;
  private Workspace workspace;
  private JPanel resultCounterPane;
  private JLabel resultCounterLabel;

  /**
   * the constructor passes in the library to highlight the results in
   * and the results to highlight.  if results is null, the tree is built
   * fully collapsed with no highlights.
   * @param library the library to highlight the results in
   * @param results the results to highlight
   */
  public ResultTreeRebuilder(PTree library, LibrarySearchResults results)
    throws IllegalActionException
  {
    super(library, results);
    this.workspace = ((CompositeEntity)library.getModel().getRoot()).workspace();
    /*resultCounterPane = new JPanel();
    resultCounterPane.setPreferredSize(new Dimension(20, 200));
    resultCounterPane.setLayout(new BorderLayout());*/
  }

  /**
   * this method allows the search results to be updated in the panel
   * @param results the results to update to
   */
  public void update(LibrarySearchResults results) throws IllegalActionException
  {
    this.results = results;
    int resultcount;
    if(results == null || results.size() == 0)
    {
      resultcount = 0;
    }
    else
    {
      resultcount = results.size();
    }
    
    clearTree();
    this.removeAll();
    
    //add the results if there are any
    if(resultcount > 0)
    { //add the results tree.
      this.newRoot = new EntityLibrary(workspace);
      try
      {
        newRoot.setName("Search Results");
      }
      catch(IllegalActionException iae)
      {
        throw iae;
      }
      catch(NameDuplicationException nde)
      {
        throw new IllegalActionException("name duplication exception: " + nde);
      }
      buildResultTree();
      trimmedLibrary = new VisibleTreeModel(newRoot);
      resultsTree = new PTree(trimmedLibrary);
      JScrollPane newpane = new JScrollPane(resultsTree);
      newpane.setPreferredSize(new Dimension(200, 200));
      this.add(newpane, BorderLayout.CENTER);
      expandAll(resultsTree);
    }
    else
    { //if there are no results, just add the library back
      this.add(new JScrollPane(library), BorderLayout.CENTER);
    }
    
    //add the search results counter stuff
    resultCounterPane = new JPanel();
    resultCounterPane.setLayout(new BorderLayout());
    resultCounterLabel = new JLabel(resultcount + " results found.");
    resultCounterPane.removeAll();
    resultCounterPane.add(resultCounterLabel, BorderLayout.CENTER);
    this.add(resultCounterPane, BorderLayout.SOUTH);
    
    this.repaint();
    this.validate();
  }
  
  /**
   * clears the tree of any items contained by newRoot
   */
  private void clearTree()
    throws IllegalActionException
  {
    if(newRoot != null && newRoot.entityList() != null)
    {
      for(Iterator i = newRoot.entityList().iterator(); i.hasNext();)
      {
        ComponentEntity ce = (ComponentEntity)i.next();
        try
        {
          ce.setContainer(null);
          executeChangeRequest(ce, newRoot);
        }
        catch(IllegalActionException iae)
        {
          throw new IllegalActionException("Cannot remove entity from search " +
            "result tree");
        }
        catch(NameDuplicationException nde)
        {
          throw new IllegalActionException("cannot set container because the name " +
            "already exists: " + nde);
        }
        catch(IOException ioe)
        {
          throw new IllegalActionException("IO Exception while removing entity " +
            " search result tree: " + ioe);
        }
        catch(Exception e)
        { //do nothing...just go onto the next node
        }
      }
    }
  }

  /**
   * build the result tree
   */
  private void buildResultTree() throws IllegalActionException
  {
    for(int i=0; i<results.size(); i++)
    { //iterate over the treepaths in results
      TreePath path = results.getTreePath(i);
      NamedObj clone = null;
      NamedObj pathComponent = (NamedObj)path.getPathComponent(
        path.getPathCount() - 1); //get the leaf node of the result path

      NamedObj currentEntity = cloneEntity(pathComponent);

      for(int j=path.getPathCount() - 2; j>=0; j--)
      {
        EntityLibrary newEntity = (EntityLibrary)getDeepEntity(
          (CompositeEntity)path.getPathComponent(j), newRoot);
        CompositeEntity pathEntity = (CompositeEntity)path.getPathComponent(j);
        if(newEntity == null && !pathEntity.getName().equals("actor library"))
        {
          newEntity = copyCompositeEntity(pathEntity);
        }

        try
        {
          if(j == 0)
          { //the root node
            if(getDeepEntity(currentEntity, newRoot) == null)
            {
              setContainer(newRoot, currentEntity);
              executeChangeRequest(currentEntity, newRoot);
            }
          }
          else
          { //all the others
            if(getDeepEntity(currentEntity, newEntity) == null)
            {
              if(currentEntity instanceof CompositeActor)
              {
                ((CompositeActor)currentEntity).setContainer(newEntity);
              }
              else
              {
                if(currentEntity.getContainer() != null)
                {
                  setContainer(newEntity, currentEntity);
                  executeChangeRequest(currentEntity, newEntity);
                }
              }
            }
          }
        }
        catch(IllegalActionException iae)
        {
          throw new IllegalActionException("cannot build search result tree: " + iae);
        }
        catch(NameDuplicationException nde)
        {
          //FIXME: ignore this for now.  I'm not sure why this works
          //throw new IllegalActionException("cannot add entity to the search " +
          //  "result container because the name already exists: " + nde);
        }
        catch(IOException ioe)
        {
          throw new IllegalActionException("IO Exception: " + ioe);
        }
        catch(Exception e)
        {
          //FIXME: These exceptions are being thrown from the changeRequest.
          //I'm not sure why they are being thrown or why I can't catch
          //them as an IllegalActionException so......
          //do nothing.  just go onto the next path..
          System.out.println("EXCEPTION CAUGHT: " + e.getMessage());
        }

        currentEntity = newEntity;
      }
    }
  }

  /**
   * puts entity in container
   */
  private static void setContainer(CompositeEntity container, NamedObj entity)
    throws NameDuplicationException, IllegalActionException
  {
    if(entity instanceof Attribute)
    {
      ((Attribute)entity).setContainer(container);
    }
    else if(entity instanceof CompositeActor)
    {
      ((CompositeActor)entity).setContainer(container);
    }
    else
    {
      ((ComponentEntity)entity).setContainer(container);
    }
  }

  /**
   * search entityToSearch for entity.  this searches all entities contained
   * by entityToSearch (deep search).  returns null if entity does not exist
   * in entityToSearch
   */
  private static NamedObj getDeepEntity(NamedObj entity, 
    CompositeEntity entityToSearch)
  {
    if(entityToSearch instanceof CompositeActor)
    {
      return null;
    }
    
    for(Iterator i = entityToSearch.entityList().iterator(); i.hasNext();)
    { //search each enity in the entityToSearch
      NamedObj obj = (Entity)i.next();
      if(obj.getName().equals(entity.getName()))
      {
        return obj;
      }
      else
      {
        if(obj instanceof CompositeEntity)
        { //if we find another compositeentity, go into it and search
          NamedObj returnEntity = getDeepEntity(entity, (CompositeEntity)obj);
          if(returnEntity != null)
          { //if we find something, return it back through the recursion
            return returnEntity;
          }
        }
      }
    }
    
    return null; //no results found
  }

  /**
   * copy the entity in name only unless the deepCopy flag is true, then
   * copy the entity in name and copy it's contents as well
   */
  private EntityLibrary copyCompositeEntity(CompositeEntity entity) 
    throws IllegalActionException
  {
    try
    {
      return new EntityLibrary(new CompositeEntity(this.workspace),
        entity.getName());
    }
    catch(IllegalActionException iae)
    {
      throw new IllegalActionException("cannot copy composite entity: " + iae);
    }
    catch(NameDuplicationException nde)
    {
      throw new IllegalActionException("cannot set container because the name " +
        "already exists: " + nde);
    }
  }

  /**
   * request a change to the model
   */
  private static void executeChangeRequest(NamedObj changedEntity,
    NamedObj targetEntity)
    throws NameDuplicationException, IllegalActionException, IOException
  {
    StringWriter buffer = new StringWriter();
    changedEntity.exportMoML(buffer, 1);
    ChangeRequest request = new MoMLChangeRequest(changedEntity, targetEntity,
      buffer.toString());
    targetEntity.requestChange(request);
  }

  /**
   * clone the entity into the new workspace.
   */
  private NamedObj cloneEntity(NamedObj entity)
    throws IllegalActionException
  {
    try
    {
      return (NamedObj)entity.clone(this.workspace);
    }
    catch(java.lang.CloneNotSupportedException cnse)
    {
      throw new IllegalActionException("clone not supported: " + cnse);
    }
  }

  /**
   * A factory that creates the searcher to search the library
   */
  public static class Factory extends LibrarySearchResultPaneFactory
  {
    /** Create an factory with the given name and container.
     *  @param container The container.
     *  @param name The name of the entity.
     *  @exception IllegalActionException If the container is incompatible
     *   with this attribute.
     *  @exception NameDuplicationException If the name coincides with
     *   an attribute already in the container.
     */
    public Factory(NamedObj container, String name)
            throws IllegalActionException, NameDuplicationException
    {
      super(container, name);
    }

    /**
     *  creates a ResultTreeBuilder and returns it.
     *  @param _libraryModel the model containing the actor library
     *  @return A new LibraryPane that displays the library
     */
    public LibrarySearchResultPane createLibrarySearchResultPane(PTree library,
      LibrarySearchResults results) throws IllegalActionException
    {
      return new ResultTreeRebuilder(library, results);
    }
  }
}
