/**
 *    '$RCSfile: SelectedEcoGridService.java,v $'
 *
 *     '$Author: tao $'
 *       '$Date: 2004/10/14 18:27:52 $'
 *   '$Revision: 1.3 $'
 *
 *  For Details: http://kepler.ecoinformatics.org
 *
 * Copyright (c) 2003 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */


package org.ecoinformatics.seek.ecogrid;

import java.util.Vector;

/**
 * This class extends from EcoGridService, it contains selected status from
 * user interface
 * @author Jing Tao
 *  
 */

public class SelectedEcoGridService extends EcoGridService 
{
 
  private SelectedServiceName selectedServiceName          = null;
  private SelectedDocumentType[] selectedDcoumentTypeList  = null;
  /**
   * Default constructor and inits selected service name and document type array
   */
  public SelectedEcoGridService() 
  {
    super();
    initSeletedServiceName();
    initSelectedDocumentTypeList();
  }//SelectedEcoGridService
  
  /**
   * Copy Constructor, copy a EcoGridService and inited selected service name
   * and selected document type array
   * @param myService EcoGridService
   */
  public SelectedEcoGridService(EcoGridService myService)
  {
    super(myService);
    initSeletedServiceName();
    initSelectedDocumentTypeList();
  }
  
  /*
   * Method to initial Seletect service name
   */
  private void initSeletedServiceName()
  {
    selectedServiceName = new SelectedServiceName();
    selectedServiceName.setServiceName(super.getServiceName());
    selectedServiceName.setIsSelected(SelectedObjectInterface.DEFAULTSELECTIONSTATUS);
  }//initSelectedServiceName
  
  /*
   * Method to initial select document type array
   */
  private void initSelectedDocumentTypeList()
  {
    DocumentType[] documentTypeList = super.getDocumentTypeList();
    if (documentTypeList != null)
    {
      int length = documentTypeList.length;
      selectedDcoumentTypeList = new SelectedDocumentType[length];
      for (int i=0; i<length; i++)
      {
        DocumentType documentType = documentTypeList[i];
        SelectedDocumentType selectedDcoumetType = new SelectedDocumentType
                                     (documentType, SelectedObjectInterface.DEFAULTSELECTIONSTATUS);
        selectedDcoumentTypeList[i] = selectedDcoumetType;
      }
    }//if
  }//initSeletedCoumentTypeList
  
  /**
   * Method to get selected service name
   * @return SelectedServiceName
   */
  public SelectedServiceName getSelectedServiceName()
  {
    return this.selectedServiceName;
  }//getSelectedServieName
  
  /**
   * Method to set selected service name
   * @param selectedServiceName SelectedServiceName
   */
  public void setSelectedServiceName(SelectedServiceName selectedServiceName)
  {
    this.selectedServiceName = selectedServiceName;
  }//setSelectedServiceName
  
  /**
   * Method to get selectedDocumentType list
   * @return SelectedDocumentType[]
   */
  public SelectedDocumentType[] getSelectedDocumentTypeList()
  {
    return this.selectedDcoumentTypeList;
  }//getSelectedDocumentTypeList
  
  /**
   * Method to set selected docuemnt type list
   * @param selectedDocumentTypeList SelectedDocumentType[]
   */
  public void setSelectedDocumentTypeList(SelectedDocumentType[] selectedDocumentTypeList)
  {
    this.selectedDcoumentTypeList = selectedDocumentTypeList;
  }//setSelectedDocumentTypeList
  
  /**
   * Method to transfer a vecotr of ecogrid service to default selected ecogrid
   * service vector. This is a utility method
   * @param serviceVector Vector
   * @return Vector
   */
  public static Vector transferServiceVectToDefaultSelectedServiceVect(Vector serviceVector)
  {
    Vector selectedServiceVector = new Vector();
    if (serviceVector == null)
    {
      return selectedServiceVector;
    }//if
    int size = serviceVector.size();
    for (int i=0; i<size; i++)
    {
      EcoGridService service = (EcoGridService)serviceVector.elementAt(i);
      SelectedEcoGridService selectedService = new SelectedEcoGridService(service);
      selectedServiceVector.add(selectedService);
    }//for
    return selectedServiceVector;
  }//transferServiceVectToDefaultSelectedServiceVect
  
}//SelectedEcoGridService