/**
 *    '$RCSfile: ServicesDisplayPanel.java,v $'
 *
 *     '$Author: tao $'
 *       '$Date: 2004/10/16 00:38:53 $'
 *   '$Revision: 1.5 $'
 *
 *  For Details: http://kepler.ecoinformatics.org
 *
 * Copyright (c) 2004 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

package org.ecoinformatics.seek.ecogrid;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import java.awt.Color;
import javax.swing.JTable;
import java.util.Vector;
import javax.swing.table.TableColumn;
import javax.swing.JFrame;
import java.awt.BorderLayout;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import org.ecoinformatics.util.DebugPrinter;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import javax.swing.JCheckBox;

/**
 * This class will represents a panel which will display services
 * @author Jing Tao
 *  
 */

public class ServicesDisplayPanel extends JScrollPane 
{
  public static final int LABELPREFERWIDTH   = 180; 
  public static final int CELLHEIGHT         = 30;
  public static final int CELLPREFERREDWIDTH = 200;
  public static final int CELLMINIWIDTH      = 200;
  public static final int CELLMAXWIDTH       = 300;
  public static final String SERVICENAMECOL  = "Service Name";
  public static final String LOCATIONCOL     = "Location";
  public static final String DOCUMENTTYPECOL = "Document Type";
  
  private static final boolean ROWSELECTION    = false;
  private static final boolean CELLSELECTION   = false;
  private static final boolean COLUMNSELECTION = false;
  
  private EcoGridServicesController serviceController = null;
  private ServicesDisplayTableModel tableModel        = null;
  private JTable table                                = null;
  private CheckBoxTableCellRenderer checkBoxRenderer  = null;
  private TableTableCellRenderer tableRenderer        = null; 
  private DebugPrinter dbg                            = DebugPrinter.getInstance();
  
  // If we need modify columns in table, we need modify this array
  // and getValueAt method in ServicesDisplayTableModel class 
  public static final String[] HEADNAME = {SERVICENAMECOL, DOCUMENTTYPECOL};
  
  /**
   * Constructor of ServicesDisplayPane
   * @param serviceController EcoGridServicesController
   */
  public ServicesDisplayPanel(EcoGridServicesController serviceController) 
  {
    this.serviceController = serviceController;
    init();
  }//ServiceDispalyPane
  
  /*
   * Method to init the pane
   */
  private void init()
  {
    this.getViewport().setBackground(Color.WHITE);
    
    Vector unSelectedserviceList = serviceController.getServicesList();
    //transfer to selectedSerive list(object is SelectedEcoGridService now)
    Vector selectedServiceList    = 
        SelectedEcoGridService.transferServiceVectToDefaultSelectedServiceVect(unSelectedserviceList); 
    tableModel = new ServicesDisplayTableModel(selectedServiceList, HEADNAME);
    table      = new JTable(tableModel);
    table.setRowSelectionAllowed(ROWSELECTION);
    table.setColumnSelectionAllowed(COLUMNSELECTION);
    table.setCellSelectionEnabled(CELLSELECTION);
    Vector rowHeightFactorList = tableModel.getRowHeightFactor();
    initRowHeight(rowHeightFactorList);
    initColumnWidth();
   
    //first col(service name render use checkbox renderer
    //second col(location) use default rendered
    //third col(doctype) use table rendered
    checkBoxRenderer     = new CheckBoxTableCellRenderer(selectedServiceList, 
                                      CheckBoxTableCellRenderer.DEFAUTTOPROW);
    tableRenderer        = new TableTableCellRenderer(selectedServiceList);
    TableColumn serviceNameCol = table.getColumn(SERVICENAMECOL);
    serviceNameCol.setCellRenderer(checkBoxRenderer);
    serviceNameCol.setCellEditor(new CheckBoxTableCellEditor(new JCheckBox(), 
                   selectedServiceList, CheckBoxTableCellRenderer.DEFAUTTOPROW));
    TableColumn docTypeCol = table.getColumn(DOCUMENTTYPECOL);
    docTypeCol.setCellRenderer(tableRenderer);
    docTypeCol.setCellEditor(new TableTableCellEditor(new JCheckBox(), selectedServiceList));
  
    this.getViewport().add(table);
  }//init
  
  
  /* 
   * This method picks good column sizes.
   */
 private void initColumnWidth() 
 {
    TableColumn column = null;
    for (int i = 0; i < tableModel.getColumnCount(); i++) 
    {
      column = table.getColumnModel().getColumn(i);
      column.setPreferredWidth(CELLPREFERREDWIDTH);
      //column.setMaxWidth(CELLMAXWIDTH);
      //column.setMinWidth(CELLMINIWIDTH);
    }//for
 }//initColumnSizes
 
  /*
   * Initial the row height base on given row height factor. If the factor is
   * null, then table will have unique height
   */
  private void initRowHeight(Vector factorList)
  {
    if (factorList != null)
    {
      int rowNumber = table.getRowCount();
      int listSize  = factorList.size();
      if (rowNumber <= 0)
      {
        table.setRowHeight(CELLHEIGHT);
      }//if
      else
      {
        for (int i=0; i<rowNumber; i++)
        {
          if (i<listSize)
          {
            int factor = ((Integer)factorList.elementAt(i)).intValue();
            dbg.print("The factor for row " + i + " is " +factor, 2);
            table.setRowHeight(i, factor*CELLHEIGHT);
          }
          else
          {
            table.setRowHeight(i, CELLHEIGHT);
          }
        }//for
      }//else
    }//if
    else
    {
      table.setRowHeight(CELLHEIGHT);
    }//
  }//initRowHeight

  
  public static void main(String[] args)
  {
    int width  = 600;
    int height = 140;
    EcoGridQueryServicesController controller = new EcoGridQueryServicesController();
    ServicesDisplayPanel serviceDisplayPane = new ServicesDisplayPanel(controller);
    //set up a frame
    JFrame frame = new JFrame("SwingApplication");
    frame.setSize(width, height);
    frame.getContentPane().add(serviceDisplayPane, BorderLayout.CENTER);
    //Finish setting up the frame, and show it.
    frame.addWindowListener(new WindowAdapter() 
    {
      public void windowClosing(WindowEvent e) 
      {
        System.exit(0);
      }
    });
    frame.setVisible(true);

  }//main
  
 

}//ServicesDisplayPane