/**
 *    '$RCSfile: SimpleChoiceLibrarySearcher.java,v $'
 *
 *     '$Author: berkley $'
 *       '$Date: 2004/10/13 23:32:47 $'
 *   '$Revision: 1.3 $'
 *
 *  For Details: http://kepler.ecoinformatics.org
 *
 * Copyright (c) 2004 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

package org.ecoinformatics.seek.ecogrid;


import javax.swing.*;
import javax.swing.tree.*;
import java.awt.*;
import java.awt.event.*;

import java.util.*;

import ptolemy.moml.ErrorHandler;
import ptolemy.moml.MoMLParser;
import ptolemy.kernel.CompositeEntity;
import ptolemy.kernel.util.*;
import ptolemy.vergil.tree.*;

/**
 * A simple searcher that mixes title search and annotation search.
 * Note that this is really a meta-searcher: it delegates to the
 * Simple and Ont library searchers; and combines results when needed.
 */
public class SimpleChoiceLibrarySearcher extends LibrarySearcher {

    /**
     * Constructor
     * @param search requires a choice gui pane.
     */
    public SimpleChoiceLibrarySearcher(JTree library, SimpleChoiceLibrarySearchGUIPane searchPane) {
	super(library, searchPane);
    }

    
    /**
     * Search for ontology-keyword in the library
     * @param value the keyword in the ontology to search for
     */
    public LibrarySearchResults search(String value) {
	SimpleChoiceLibrarySearchGUIPane pane = (SimpleChoiceLibrarySearchGUIPane)searchPane;

	// assign the new results of the search
	results = new LibrarySearchResults();
	if(value.trim().equals(""))
	    return results;

	// figure out what kind of search we need to do
	if(pane.isTextSearchType() && !pane.isKeywordSearchType()) {
	    // do the SimpleSearch
	    LibrarySearcher searcher = new SimpleLibrarySearcher(library, searchPane);
	    results = searcher.search(value);
	}
	else if(pane.isKeywordSearchType() && !pane.isTextSearchType()) {
	    // do the OntSearch
	    LibrarySearcher searcher = new OntLibrarySearcher(library, searchPane);
	    results = searcher.search(value);
	}
	else if(pane.isTextSearchType() && pane.isKeywordSearchType()) {
	    // do both: this might need optimizing at some point
	    LibrarySearcher searcher;
	    searcher = new SimpleLibrarySearcher(library, searchPane);
	    results = searcher.search(value); 
	    searcher = new OntLibrarySearcher(library, searchPane);
	    Iterator iter = searcher.search(value).iterator();	   
	    while(iter.hasNext()) {
		TreePath p = (TreePath)iter.next();
		if(!results.contains(p)) {
		    results.add(p);
		}
	    }
	}
	
	return results;
    }


    /**
     * provides any initialization needed prior to searching.  It is
     * called when the class is constructed.  Note that any local
     * variables of extending classes should be initialized in init
     * since it is called be the constructor of the super class
     * (LibrarySearcher).
     */
    protected void init() {
    }


    /**
     * A factory that creates the searcher to search the library
     */
    public static class Factory extends LibrarySearcherFactory {
	/** 
	 * Create a factory with the given name and container.
	 * @param container The container.
	 * @param name The name of the entity.
	 * @exception NameDuplicationException If the name coincides
	 * with an attribute already in the container
	 */ 
	public Factory(NamedObj container, String name)
	    throws IllegalActionException, NameDuplicationException {
	    super(container, name);
	}

	/**
	 * Creates an Ontology Library Searcher and returns it.
	 * @param library The model containing the actor library.
	 * @return A new Library Searcher.
	 */
	public LibrarySearcher createLibrarySearcher(JTree library, LibrarySearchPane searchPane) {
	    return new SimpleChoiceLibrarySearcher(library, (SimpleChoiceLibrarySearchGUIPane)searchPane);
	}


    } // Factory



} // OntLibrarySearcher



