/**
 *    '$RCSfile: TabbedLibraryPane.java,v $'
 *
 *     '$Author: berkley $'
 *       '$Date: 2004/10/13 21:55:10 $'
 *   '$Revision: 1.28 $'
 *
 *  For Details: http://kepler.ecoinformatics.org
 *
 * Copyright (c) 2004 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */
package org.ecoinformatics.seek.ecogrid;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.BorderLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTree;
import javax.swing.ActionMap;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.InputMap;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.tree.TreeModel;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

import ptolemy.gui.CancelException;
import ptolemy.gui.MessageHandler;
import ptolemy.kernel.util.IllegalActionException;
import ptolemy.kernel.util.NameDuplicationException;
import ptolemy.kernel.util.NamedObj;
import ptolemy.kernel.Entity;
import ptolemy.vergil.tree.EntityTreeModel;
import ptolemy.vergil.tree.PTree;
import ptolemy.kernel.ComponentEntity;
import ptolemy.moml.MoMLParser;
import ptolemy.moml.ErrorHandler;
import ptolemy.actor.gui.Configuration;

import java.util.Vector;
import java.util.List;
import java.util.EmptyStackException;
import java.util.Stack;

import edu.ucsb.nceas.configxml.ConfigXML;

import org.ecoinformatics.seek.sms.AnnotationEngine;


/**
 * A simple pane used to display the contents of the library tree using a tabbed pane
 * to provide additional resources in the library.
 *
 * @author  Matt Jones
 * @version $Id: TabbedLibraryPane.java,v 1.28 2004/10/13 21:55:10 berkley Exp $
 * @since Kepler 1.0
 */
public class TabbedLibraryPane extends LibraryPane
{
    /**
     * Construct a new library pane for displaying the tree of actors and other
     * resourses that can be dragged to the graph editor.
     *
     * @param _libraryModel the model containing the library to be displayed
     */
    public TabbedLibraryPane(EntityTreeModel _libraryModel, Configuration config)
    {
        super();

	// ADDED BY: bowers
	// call the annotation engine here and construct the new library model on the fly
	AnnotationEngine eng = AnnotationEngine.instance();
	_libraryModel = eng.buildDefaultActorLibrary(_libraryModel);


        this.setLayout(new BorderLayout());
        _library = new PTree(_libraryModel);
        _library.setRootVisible(false);
        //_library.setBackground(BACKGROUND_COLOR);



        //create the library search results panel from the config
        LibrarySearchResultPaneFactory SRPfactory =
          (LibrarySearchResultPaneFactory)
          config.getAttribute("librarySearchResultPane");
        if (SRPfactory != null) {
          try {
            _libraryPanel =
              SRPfactory.createLibrarySearchResultPane(_library, null);
            if (_libraryPanel == null) {
              System.out.println("error: _libraryPanel is null.  " +
                "This " +
                "problem can be fixed by adding a librarySearcherFactory " +
                "property in the configuration.xml file.");
            }
          } catch(IllegalActionException iae) {
            System.out.println("Error displaying search results: " + iae);
          }
        } else {
          System.out.println("error: librarySearchResultPane.factory is " +
                "null.  This " +
                "problem can be fixed by adding a librarySearcherFactory " +
                "property in the configuration.xml file.");
        }

        //create the search GUI pane
        LibrarySearchGUIPaneFactory LSGfactory =
          (LibrarySearchGUIPaneFactory)
          config.getAttribute("librarySearchGUIPane");
        if (LSGfactory != null) {
          _searchPane =
            LSGfactory.createLibrarySearchGUIPane(searchButtonHandler);
          if (_searchPane == null) {
            System.out.println("error: _searchPane is null.  " +
              "This " +
              "problem can be fixed by adding a librarySearchGUIPane " +
              "property in the configuration.xml file.");
          }
        } else {
          System.out.println("error: LibrarySearchGUIPane.factory is " +
                "null.  This " +
                "problem can be fixed by adding a librarySearchGUIPane " +
                "property in the configuration.xml file.");
        }


        //create the librarySearcher from the configuration
        LibrarySearcherFactory LSfactory = (LibrarySearcherFactory)
          config.getAttribute("librarySearcherFactory");
        if (LSfactory != null) {
            librarySearcher = LSfactory.createLibrarySearcher(_library, _searchPane);
            if (librarySearcher == null) {
              System.out.println("error: librarySearcher is null.  This " +
                "problem can be fixed by adding a librarySearcherFactory " +
                "property in the configuration.xml file.");
            }
        } else {
          System.out.println("error: librarySearcher.factory is null.  This " +
                "problem can be fixed by adding a librarySearcherFactory " +
                "property in the configuration.xml file.");
        }




        JPanel librarySearchPanel = new JPanel();
        librarySearchPanel.setLayout(topLayout);
        librarySearchPanel.setMinimumSize(new Dimension(200, 500));
        librarySearchPanel.setPreferredSize(new Dimension(200, 500));
        //create and add the search panel to the library

        _libraryPanel.setPreferredSize(new Dimension(200, 400));
        librarySearchPanel.add(_searchPane, BorderLayout.NORTH);
        librarySearchPanel.add(_libraryPanel, BorderLayout.CENTER);

        searchPartPanel.setPreferredSize(new Dimension(200, 100));
        searchPartPanel.setMinimumSize(new Dimension(100, 75));


        _libraryTabPane = new JTabbedPane(JTabbedPane.TOP);
        _libraryTabPane.add("Actors", librarySearchPanel);

        try {
             EcoGridQueryServicesController controller = new EcoGridQueryServicesController();
             DatasetPanel _datasetPanel = new DatasetPanel(controller);
          
            //_datasetPanel.setEcoGridServiceController(controller);

            _libraryTabPane.add("Data", _datasetPanel);
        } catch (Exception e) {
            try {
                MessageHandler.warning("Could not create data pane.", e);
            } catch (Exception ce) {
                // Do nothing
            }
        }

        this.add(_libraryTabPane, BorderLayout.CENTER);
    }

    /**
     * A factory that creates the library panel for the editors.
     */
    public static class Factory extends LibraryPaneFactory
    {
        /** Create an factory with the given name and container.
         *  @param container The container.
         *  @param name The name of the entity.
         *  @exception IllegalActionException If the container is incompatible
         *   with this attribute.
         *  @exception NameDuplicationException If the name coincides with
         *   an attribute already in the container.
         */
        public Factory(NamedObj container, String name)
                throws IllegalActionException, NameDuplicationException
        {
            super(container, name);
        }

        /** Create a library pane that displays the given library of actors.
         *
         *  @param _libraryModel the model containing the actor library
         *  @return A new LibraryPane that displays the library
         */
        public LibraryPane createLibraryPane(EntityTreeModel _libraryModel, Configuration config)
        {
                TabbedLibraryPane pane = new TabbedLibraryPane(_libraryModel, config);
                return pane;
        }
    }

    /**
     * class to handle the search button
     */
    public class SearchButtonActionHandler implements ActionListener
    {
      public void actionPerformed(ActionEvent e)
      {
        try
        {
          _libraryPanel.update(librarySearcher.search(_searchPane.getSearchTerm()));
        }
        catch(IllegalActionException iae)
        {
          //FIXME: handle this exception correctly
          System.out.println("Error searching: " + iae);
          iae.printStackTrace();
        }

      }
    }

    ///////////////////////////////////////////////////////////////////
    ////                         private variables                 ////

    // FIXME: Comments are needed on all these.
    // FIXME: Need to be in alphabetical order.

    // NOTE: should be somewhere else?
    // Default background color is a light grey.
    private static Color BACKGROUND_COLOR = new Color(0xe5e5e5);

    private PTree _library;
    private LibrarySearchResultPane _libraryPanel;
    private JScrollPane _dataScrollPane;
    private JTabbedPane _libraryTabPane;
    private JPanel searchPartPanel = new JPanel();
    private LibrarySearcher librarySearcher;
    private Configuration config;
    private BorderLayout topLayout = new BorderLayout();
    private LibrarySearchPane _searchPane;
    private SearchButtonActionHandler searchButtonHandler =
      new SearchButtonActionHandler();
}
