/**
 *    '$RCSfile: NamedObjId.java,v $'
 *
 *     '$Author: berkley $'
 *       '$Date: 2004/10/13 21:55:10 $'
 *   '$Revision: 1.2 $'
 *
 *  For Details: http://kepler.ecoinformatics.org
 *
 * Copyright (c) 2003 The Regents of the University of California.
 * All rights reserved.
 * 
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 * 
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

package org.ecoinformatics.seek.util;

import ptolemy.kernel.util.AbstractSettableAttribute;
import ptolemy.kernel.util.NamedObj;
import ptolemy.kernel.util.Workspace;
import ptolemy.kernel.util.IllegalActionException;
import ptolemy.kernel.util.NameDuplicationException;
import ptolemy.kernel.util.ValueListener;
import ptolemy.kernel.util.Settable;
import ptolemy.kernel.util.Settable.Visibility;

import java.util.Vector;

public class NamedObjId extends AbstractSettableAttribute
{
  /** container for the value */
  private String value = null;
  /** container for valueListeners assigned to this attribute */
  private Vector valueListeners = new Vector();
  
  /**
   * Constructor
   */
  public NamedObjId()
  {
    super();
  }
  
  /**
   * Constructor
   */
  public NamedObjId(NamedObj container, String name) throws IllegalActionException,
    NameDuplicationException
  {
    super(container, name);
  }
  
  /**
   * Constructor
   */
  public NamedObjId(Workspace workspace)
  {
    super(workspace);
  }
  
  /**
   * returns the default expression which is null
   */
  public String getDefaultExpression()
  {
    return null;
  }
  
  /**
   * set the value of this id
   */
  public void setExpression(String expression)
  {
    // System.out.println("setting NamedObjId to " + expression);
    value = expression;  //set the value
    
    for(int i=0; i<valueListeners.size(); i++)
    { //notify any listeners of the change
      ValueListener listener = (ValueListener)valueListeners.elementAt(i);
      listener.valueChanged(this);
    }
  }
  
  /**
   * return the value of the id
   */
  public String getExpression()
  {
    return value;
  }
  
  /**
   * add a valueListener
   */
  public void addValueListener(ValueListener listener)
  {
    valueListeners.add(listener);
  }
  
  /**
   * NamedObjIds should be invisible to the user
   */
  public Settable.Visibility getVisibility()
  {
    return NONE;
  }
  
  /**
   * this method does not change the visibility.  NamedObjId should only be
   * invisible
   */
  public void setVisibility(Settable.Visibility visibility)
  {
    //do nothing....we don't want the visibility getting changed
  }
  
  /**
   * remove the indicated listener
   */
  public void removeValueListener(ValueListener listener)
  {
    valueListeners.remove(listener);
  }
  
  /**
   * validate the expression. 
   */
  public void validate()
  {
    //don't need to do anything here
  }

  public boolean equals(Object obj) 
  {
      if(!(obj instanceof NamedObjId)) 
	  return false; 
      NamedObjId objId = (NamedObjId)obj;
      String str = objId.getExpression();
      if(this.getExpression() == null) {
	  if(str != null)
	      return false;
	  return true;
      }
      return this.getExpression().equals(objId.getExpression());
  }
}
