/**
 * Copyright (c) 2004 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

package org.sdm.spa;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;

import ptolemy.actor.TypedAtomicActor;
import ptolemy.actor.TypedIOPort;
import ptolemy.data.StringToken;
import ptolemy.data.type.BaseType;
import ptolemy.kernel.CompositeEntity;
import ptolemy.kernel.util.IllegalActionException;
import ptolemy.kernel.util.NameDuplicationException;
public class GISequence extends TypedAtomicActor {

	public GISequence(CompositeEntity aContainer, String aName)
		throws IllegalActionException, NameDuplicationException {

		super(aContainer, aName);
		result = new TypedIOPort(this, "result", false, true);
		result.setTypeEquals(BaseType.STRING);
		WP_GI = new TypedIOPort(this, "GI", true, false);
		WP_GI.setTypeEquals(BaseType.STRING);
	}

	///////////////////////////////////////////////////////////////////
	////                     ports and parameters                  ////

	// -- Part of the Actor

	public TypedIOPort result;
	public TypedIOPort WP_GI;

	///////////////////////////////////////////////////////////////////
	////                         public methods                    ////

	/** Initialize the PN actor.
	 *  @exception IllegalActionException If the parent class throws it.
	 */
	public void initialize() throws IllegalActionException {
		super.initialize();
		_returnValue = true;


	}


	/** Fire the actor.
	 *  @exception IllegalActionException If there is no director.
	 */
	public void fire() throws IllegalActionException {
		p_gi = ((StringToken)(WP_GI.get(0))).stringValue();
		_debug("Process:" + p_gi);
		String seq = "";
		seq = retrieve(p_gi);
		if ( seq.length() > 0) {

			result.broadcast( new StringToken(seq));
		}
	}

	/**
	 * Method promoteRegion.
	 * @param gi
	 * @param start
	 * @param end
	 * @param extra
	 * @return String
	 */
	private String promoteRegion(String gi, String start, String end, String extra) {
		String res = "";
		String seq = retrieve(gi);
		int st = Integer.parseInt(start);
		int ed = Integer.parseInt(end);
		int ex = Integer.parseInt(extra);
		int s, e;
		if (st - ex	<0) {
			s = 0;
		} else {
			s = st -ex;
		}

		if (ed + ex < seq.length()) {
			e = ed + ex;
		} else {
			e = seq.length();
		}
		//int mid = (st +ed)/2;
		//res = seq.substring(mid - ex, mid + ex);

		res = seq.substring(s,e);
		return res;
	}

	/**
	 * Method retrieve: retrieve the sequence according to the
	 * GI number. The content is between <pre> and </pre>
	 * of the url.
	 * @param gi
	 * @return String
	 */
	private String retrieve(String gi) {
		//String url_query = "http://www.ncbi.nlm.nih.gov/entrez/viewer.fcgi?db=nucleotide&cmd=Display&dopt=nucleotide_comp_nucleotide&from_uid="+gi+"&http://www.ncbi.nlm.nih.gov/entrez/viewer.fcgi?cmd=&txt=&save=&cfm=&query_key=3&db=nucleotide&extrafeat=-1&view=fasta&dispmax=20&SendTo=on&__from=&__to=&__strand=";
		String url_query =
			"http://www.ncbi.nlm.nih.gov/entrez/query.fcgi?cmd=Retrieve&db=nucleotide&list_uids="
				+ gi
				+ "&dopt=GenBank";
		String res = null;
		String content = "";
		String line;
		String sequence = "";
		StringBuffer sb =new StringBuffer();

		//Open the url and read the content.
		try {
			URL url = new URL(url_query);
			HttpURLConnection urlconn =
				(HttpURLConnection) url.openConnection();

			BufferedReader in =
				new BufferedReader(
					new InputStreamReader(urlconn.getInputStream()));
			while ((line = in.readLine()) != null) {
				sb.append(line + "\n");
				//System.out.println(content.length());
			}
			content = sb.toString();
			//System.out.println(content);

			String tag1 = "<pre>";
			String tag2 = "</pre>";
			int i = content.indexOf(tag1);

			int j = content.indexOf(tag2);

			content = content.substring(i + tag1.length(), j);

			//Now we have the data in genbank file format
			//To extract the sequence next;
			//Those are between Origina and //
			tag1 = "ORIGIN";
			tag2 = "//";
			i = content.indexOf(tag1);
			content = content.substring(i + tag1.length());

			sb = new StringBuffer();

			//Extract the sequence
			BufferedReader bf =
				new BufferedReader(
					new InputStreamReader(
						new ByteArrayInputStream(content.getBytes())));

			try {
				while (((line = bf.readLine()) != null)) {
					if (line.trim().length() > 10) {
						sb.append(line.substring(10));
					}

				}

				sequence = sb.toString();
			} catch (IOException e) {
				e.printStackTrace();
			}

		} catch (Exception e) {
		}

		//res = sequence.replaceAll(" ", "");
		res = Util.remove(sequence, ' ');

		return res;

	}


	/** Post fire the actor. Return false to indicated that the
	 *  process has finished. If it returns true, the process will
	 *  continue indefinitely.
	 */
	public boolean postfire() {
		return true;
	}

	///////////////////////////////////////////////////////////////////
	////                         private variables                 ////

	// -- Get private copies of the parameters
	private String p_gi;

	private boolean _returnValue = true;
	//private boolean toCache = true;

}
