/* An actor for connecting to SRB.

@Copyright (c) 2002-2003 The Regents of the University of California.
All rights reserved.

Permission is hereby granted, without written agreement and without
license or royalty fees, to use, copy, modify, and distribute this
software and its documentation for any purpose, provided that the
above copyright notice and the following two paragraphs appear in all
copies of this software.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

                                                PT_COPYRIGHT_VERSION 2
                                                COPYRIGHTENDKEY
*/

package org.srb;

import ptolemy.actor.TypedAtomicActor;
import ptolemy.actor.TypedIOPort;
import ptolemy.actor.lib.Source;
import ptolemy.data.Token;
import ptolemy.data.expr.Parameter;
import ptolemy.data.ArrayToken;
import ptolemy.data.IntToken;
import ptolemy.data.ObjectToken;
import ptolemy.data.StringToken;
import ptolemy.data.UnsignedByteToken;
import ptolemy.data.type.ArrayType;
import ptolemy.data.type.BaseType;
import ptolemy.data.type.Type;
import ptolemy.kernel.CompositeEntity;
import ptolemy.kernel.util.*;
import ptolemy.gui.GraphicalMessageHandler;
import ptolemy.gui.MessageHandler;
import ptolemy.actor.gui.BrowserLauncher;
import ptolemy.kernel.util.StringAttribute;

//For java APIs
import java.sql.*;
import java.io.*;
import java.util.*;
import java.net.*;

import edu.sdsc.grid.io.local.*;
import edu.sdsc.grid.io.srb.*;
import edu.sdsc.grid.io.*;

//////////////////////////////////////////////////////////////////////////
//// SRBConnect
/** An SRB connection actor.
 *
@author Bing Zhu & Efrat Jaeger
@version $Id: SRBConnect.java,v 1.2 2004/10/09 00:30:03 jaeger Exp $
@since Ptolemy II 3.0.2
*/
public class SRBConnect extends Source {

 /** Construct an actor with the given container and name.
 *  @param container The container.
 *  @param name The name of this actor.
 *  @exception IllegalActionException If the actor cannot be contained
 *   by the proposed container.
 *  @exception NameDuplicationException If the container already has an
 *   actor with this name.
 */
public SRBConnect(CompositeEntity container, String name)
        throws NameDuplicationException, IllegalActionException  {

    super(container, name);

    srbHost = new StringAttribute(this, "srbHost");
    srbPort = new StringAttribute(this, "srbPort");
    srbUserName = new StringAttribute(this, "srbUserName");
    srbPasswd = new StringAttribute(this, "srbPasswd");
    srbHomeCollection = new StringAttribute(this, "srbHomeCollection");
    srbMdasDomainHome = new StringAttribute(this, "srbMdasDomainHome");
    srbDefaultResource = new StringAttribute(this, "srbDefaultResource");

    // Set the type constraint.
    output.setTypeEquals(BaseType.GENERAL);
    output.setMultiport(true);

    _attachText("_iconDescription",
            "<svg>\n"
            + "<text x=\"0\" y=\"30\""
            + "style=\"font-size:40; fill:blue; font-family:Verdana font-style:italic; font-weight:bold\">"
            + "SRB</text>\n"
            + "</svg>\n");

}

  ///////////////////////////////////////////////////////////////////
  ////                     ports and parameters                  ////

   public StringAttribute srbHost;
   public StringAttribute srbPort;
   public StringAttribute srbUserName;
   public StringAttribute srbPasswd;
   public StringAttribute srbHomeCollection;
   public StringAttribute srbMdasDomainHome;
   public StringAttribute srbDefaultResource;

  ///////////////////////////////////////////////////////////////////
  ////                        public methods                     ////


  /** Connects to SRB and returns a connection reference.
   */
    public void fire() throws IllegalActionException {
      String _srbHost, _srbPort, _srbUserName, _srbPasswd, _srbHomeCollection, _srbMdasDomainHome, _srbDefaultResource;
      _srbHost = srbHost.getExpression();
      _srbPort = srbPort.getExpression();
      _srbUserName = srbUserName.getExpression();
      _srbPasswd = srbPasswd.getExpression();
      _srbHomeCollection = srbHomeCollection.getExpression();
      _srbMdasDomainHome = srbMdasDomainHome.getExpression();
      _srbDefaultResource = srbDefaultResource.getExpression();

      SRBAccount srbAccount = null;
      SRBFileSystem srbFileSystem = null;
      try {
         srbAccount = new SRBAccount(_srbHost, Integer.parseInt(_srbPort), _srbUserName, _srbPasswd,
                            _srbHomeCollection, _srbMdasDomainHome, _srbDefaultResource);

         for (int i=0; i<output.getWidth(); i++) {
               srbFileSystem = new SRBFileSystem( srbAccount );
               output.send(i, new ObjectToken(srbFileSystem));
         }
      } catch ( Exception ex ) {
  		  GraphicalMessageHandler.message("Could not open an SRB connection");
		  ex.printStackTrace();
      }
    }

    /** Post fire the actor. Return false to indicate that the
    * process has finished.
    */
    public boolean postfire() {
      return false;   
    }
}