/**
 *    '$RCSfile: DatasetPanel.java,v $'
 *
 *     '$Author: rspears $'
 *       '$Date: 2004/10/13 20:23:40 $'
 *   '$Revision: 1.23 $'
 *
 *  For Details: http://kepler.ecoinformatics.org
 *
 * Copyright (c) 2004 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

package org.ecoinformatics.seek.ecogrid;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.ActionMap;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultComboBoxModel;
import javax.swing.InputMap;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JTextField;
import javax.swing.KeyStroke;

import org.ecoinformatics.seek.datasource.DataCacheItem;
import org.ecoinformatics.seek.datasource.DataCacheListener;
import org.ecoinformatics.util.DebugPrinter;

import ptolemy.kernel.CompositeEntity;
import ptolemy.kernel.util.Attribute;
import ptolemy.kernel.util.IllegalActionException;
import ptolemy.kernel.util.NameDuplicationException;
import ptolemy.vergil.tree.EntityTreeModel;
import ptolemy.vergil.tree.VisibleTreeModel;

/**
 * This class will display a panel for search and search result from ecogrid
 * service.
 */
public class DatasetPanel extends JPanel implements DataCacheListener
{

    private BorderLayout topLayout = new BorderLayout();

    //declare controls
    //search part panel (above)
    private JPanel            searchPartPanel        = new JPanel();
    private JLabel            quickSearchLabel       = null;
    private QuickSearchAction quickSearchAction      = null;
    private JButton           quickSearchButton      = null;
    private JTextField        searchValueField       = null;
    private JLabel            _searchTypeLabel       = null;
    private JComboBox         _searchTypeCombo       = null;
    private JProgressBar      _progressBar           = null;
    private int               _progressListenerCount = 0;
    
    // it should be replaced by a hyperlink
    private JButton advanceSearchLabel = new JButton("Advanced Search");

    //result part panel
    private CompositeEntity resultsRoot;
    private ResultPanel resultPanel;
    private DebugPrinter dbg;
    private EcoGridServicesController searchController;
    private InputMap imap;
    private ActionMap amap;

    //constant
    private static final int EDGEGAP = 10;//GAP from container to compent
    private static final int COMPONENTGAP = 6;//GAP from compent to compent
    private static final int VERTICALEDGEGAP = 6;//GAP from container to
                                                 // compent
    private static final int VERTICALCOMPONENTGAP = 3;//GAP from compent
                                                      // tocompent
    private static final String QUERYTRANSFERCLASS = "metadataSpecificationClass";
    private static final String QUICKSEARCHLABEL = "Quick Search";
    private static final String GO = "Go";
    private static final String QUICKSEARCHACTION = "quickSearchAction";

    /**
     * Set up the pane with preference size and configuration for search
     */
    public DatasetPanel(EcoGridServicesController controller) throws Exception
    {
        searchController = controller;
        dbg = DebugPrinter.getInstance();
        init();
    }

    /**
     * Get the search text field value
     */
    public String getSearchTextFieldValue()
    {
        return searchValueField.getText();
    }

    /**
     * *** Temporary Code
     * Get the search data type (type)
     */
    public String getSearchDataSrcType()
    {
        return _searchTypeCombo.getSelectedItem().toString();
    }

    public CompositeEntity getResultRoot()
    {
      return resultsRoot;
    }

    /**
     * Initialize the panel
     *
     * @throws Exception
     */
    public void init() throws Exception
    {
        this.setLayout(topLayout);
        this.setMinimumSize(new Dimension(200, 500));
        this.setPreferredSize(new Dimension(200, 500));
        //init searchPartPanel
        initSearchPartPanel();
        //add searchPartPanel into this panel top(North)
        this.add(searchPartPanel, BorderLayout.NORTH);
        //add empty result scroll panel into this panel bottotm(South)
        resultPanel = new ResultPanel(createResultModel());
        this.add(resultPanel, BorderLayout.CENTER);
        
        _progressBar = new JProgressBar();
        this.add(_progressBar, BorderLayout.SOUTH);
        
        initKeyInputMap();

    }

    /*
     * Initial the search part panel
     */
    private void initSearchPartPanel() throws Exception
    {
        BoxLayout searchPartLayout = new BoxLayout(searchPartPanel,
                        BoxLayout.Y_AXIS);
        //searchPartPanel = new JPanel();
        searchPartPanel.setLayout(searchPartLayout);
        //add a blank panel as border
        JPanel topBlankPanel = new JPanel();
        topBlankPanel.add(Box.createVerticalStrut(VERTICALEDGEGAP));
        searchPartPanel.add(topBlankPanel);

        // quickSearchAction
        Vector ecogridServiceVector = searchController.getServicesList();
        quickSearchAction = new QuickSearchAction(ecogridServiceVector,
                                                  GO,this);

        // *** Start of Tempoarary Code
        // Add Combobox
        Vector modelVector = new Vector();
        modelVector.addElement("All");
        for (Enumeration e = quickSearchAction.transformEcoGridServiceToSearchScope().elements();e.hasMoreElements();)
        {
            SearchScope searchScope = (SearchScope)e.nextElement();
            modelVector.addElement(searchScope.getMetadataSpecification().getBriefName());
        }
        DefaultComboBoxModel model = new DefaultComboBoxModel(modelVector);
        _searchTypeLabel = new JLabel("Data Source:");
        _searchTypeCombo = new JComboBox(model);
        
        JPanel comboPanel = new JPanel();
        comboPanel.setLayout(new BoxLayout(comboPanel, BoxLayout.X_AXIS));
        comboPanel.add(Box.createHorizontalStrut(EDGEGAP));
        comboPanel.add(_searchTypeLabel);
        comboPanel.add(Box.createHorizontalStrut(COMPONENTGAP));
        
        // Combo
        quickSearchButton = new JButton(quickSearchAction);
        comboPanel.add(_searchTypeCombo);
        comboPanel.add(Box.createHorizontalStrut(EDGEGAP));
        comboPanel.add(Box.createHorizontalGlue());
        searchPartPanel.add(comboPanel);
        // *** End of Temporary Code
        
        // the panel contains label - Quick Search
        JPanel quickSearchLabelPanel = new JPanel();
        quickSearchLabelPanel.setLayout(new BoxLayout(quickSearchLabelPanel,
                        BoxLayout.X_AXIS));
        quickSearchLabelPanel.add(Box.createHorizontalStrut(EDGEGAP));
        quickSearchLabel = new JLabel(QUICKSEARCHLABEL);
        quickSearchLabelPanel.add(quickSearchLabel);
        quickSearchLabelPanel.add(Box.createHorizontalGlue());
        
        // add the quickSearchLabelPanel to searchPartPanel
        searchPartPanel.add(quickSearchLabelPanel);

        //add a bank panel between quicksearch panel and textFieldAndButton
        // panel
        JPanel blankPanelUnderQuickSearch = new JPanel();
        blankPanelUnderQuickSearch.add(Box
                        .createVerticalStrut(VERTICALCOMPONENTGAP));
        searchPartPanel.add(blankPanelUnderQuickSearch);

        // the panel contains the textfield and go button, it is has a fixed
        // height
        JPanel textFieldAndButtonPanel = new JPanel();
        textFieldAndButtonPanel.setLayout(new BoxLayout(
                        textFieldAndButtonPanel, BoxLayout.X_AXIS));
        textFieldAndButtonPanel.add(Box.createHorizontalStrut(EDGEGAP));
        searchValueField = new JTextField();
        textFieldAndButtonPanel.add(searchValueField);
        textFieldAndButtonPanel.add(Box.createHorizontalStrut(COMPONENTGAP));
        
        // quickSearchAction
        quickSearchButton = new JButton(quickSearchAction);
        textFieldAndButtonPanel.add(quickSearchButton);
        textFieldAndButtonPanel.add(Box.createHorizontalStrut(EDGEGAP));
        textFieldAndButtonPanel.add(Box.createHorizontalGlue());
        
        //add the textFieldAndButtonPanel into searchPartPanel
        searchPartPanel.add(textFieldAndButtonPanel);
        
       //add a bank panel between quicksearch panel and textFieldAndButton
        // panel
        JPanel blankPanelUnderTextField = new JPanel();
        blankPanelUnderTextField.add(Box
                        .createVerticalStrut(VERTICALCOMPONENTGAP));
        searchPartPanel.add(blankPanelUnderTextField);

        // the panel contains the advance search label. It should be replacecd
        // by a hyperlink
        /*JPanel advancedSearchPanel = new JPanel();
        advancedSearchPanel.setLayout(new BoxLayout(advancedSearchPanel,
                        BoxLayout.X_AXIS));
        advancedSearchPanel.add(Box.createHorizontalStrut(EDGEGAP));
        advancedSearchPanel.add(advanceSearchLabel);
        advancedSearchPanel.add(Box.createHorizontalGlue());
        // add this panel into searchPartPanel
        searchPartPanel.add(advancedSearchPanel);*/

        //add a bank panel between quicksearch panel and textFieldAndButton
        // panel
        JPanel blankPanelUnderAdvanceSearch = new JPanel();
        blankPanelUnderAdvanceSearch.add(Box
                        .createVerticalStrut(VERTICALEDGEGAP));
        searchPartPanel.add(blankPanelUnderAdvanceSearch);
    }

    /*
     * Init the map between keystroke and action
     */
    private void initKeyInputMap()
    {
      imap = new InputMap();
      imap.put(KeyStroke.getKeyStroke("ENTER"), QUICKSEARCHACTION);
      amap = new ActionMap();
      amap.put(QUICKSEARCHACTION, quickSearchAction);
      this.setInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT, imap);
      this.setActionMap(amap);
    }

    /**
     * Create the tree model used to contain the result set.  This is
     * displayed in the JTree on the left of the window.
     *
     * @return the EntityTreeModel that can contain the results
     */
    private EntityTreeModel createResultModel()
    {
        try {
            resultsRoot = new ResultTreeRoot("resultset");
            Attribute libraryMarker = new Attribute(resultsRoot, "_libraryMarker");
        } catch (IllegalActionException iae) {
            System.err.println("Could not create entity.");
        } catch (NameDuplicationException nde) {
            System.err.println("An entity with that name already exists.");
        }

        EntityTreeModel resultTreeModel = new VisibleTreeModel(resultsRoot);
        return resultTreeModel;
    }

    /**
     * Method to get EcoGridServicesController
     *
     * @return EcoGridServicesController
     */
    public EcoGridServicesController getSearchScope()
    {
        return this.searchController;
    }

    /**
     * Set searchController for this panel
     *
     * @param searchVector Vector
     */
    public void setEcoGridServiceController(EcoGridServicesController controller)
    {
        this.searchController = controller;
        if (quickSearchAction != null)
        {
          Vector servicesVector = controller.getServicesList();
          quickSearchAction.setSearchSerivcesVector(servicesVector);
        }
    }

    /**
     * Update the result panel after searching
     *
     * @param newTreeData CompositeEntity, the search result
     */
    public void update(EntityTreeModel newTreeData)
    {
      //resultPanel.removeAll();
      //resultPanel = new ResultPanel(newTreeData);
      //this.add(resultPanel, BorderLayout.CENTER);
      resultPanel.setTreeModel(newTreeData);

    }

    public static void main(String[] args) throws Exception
    {
        int width = 200;
        int height = 500;
        String namespace = EML2MetadataSpecification.EML200NAMESPACE;
        String className = "EML2MetadataSpecification";
        String endpoint = "http://dev.nceas.ucsb.edu:8080/ogsa/services/org/ecoinformatics/ecogrid/EcoGridQueryInterfaceLevelOneService";
        Vector endPoints = new Vector();
        endPoints.add(endpoint);
        SearchScope searchScope = new SearchScope(namespace, className, endPoints);
        Vector searchScopeVector = new Vector();
        searchScopeVector.add(searchScope);
        DatasetPanel datasetPanel = new DatasetPanel(null);
        //datasetPanel.setSearchScope(searchScopeVector);
        //set up a frame
        JFrame frame = new JFrame("SwingApplication");
        frame.setSize(width, height);
        frame.getContentPane().add(datasetPanel, BorderLayout.CENTER);
        //Finish setting up the frame, and show it.
        frame.addWindowListener(new WindowAdapter()
        {

            public void windowClosing(WindowEvent e)
            {
                System.exit(0);
            }
        });
        frame.setVisible(true);

    }

    //------------------------------------------------------------------------
    //-- DataCacheListener
    //------------------------------------------------------------------------
    
    public void complete(DataCacheItem aItem)
    {
        if (aItem == null)
        {
            _progressListenerCount++;
            _progressBar.setIndeterminate(true);
            
            quickSearchLabel.setEnabled(false);
            quickSearchButton.setEnabled(false);
            searchValueField.setEnabled(false);
            _searchTypeCombo.setEnabled(false);
            _searchTypeLabel.setEnabled(false);
            
        } else 
        {
            _progressListenerCount--;
            if (_progressListenerCount == 0) 
            {
                _progressBar.setIndeterminate(false);
                quickSearchLabel.setEnabled(true);
                quickSearchButton.setEnabled(true);
                searchValueField.setEnabled(true);
                _searchTypeCombo.setEnabled(true);
                _searchTypeLabel.setEnabled(true);
            }
        }
    }
}
