/* An actor for converting SVG files into polygon objects.



@Copyright (c) 2002-2003 The Regents of the University of California.

All rights reserved.



Permission is hereby granted, without written agreement and without

license or royalty fees, to use, copy, modify, and distribute this

software and its documentation for any purpose, provided that the

above copyright notice and the following two paragraphs appear in all

copies of this software.



IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY

FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES

ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF

THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF

SUCH DAMAGE.



THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,

INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF

MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE

PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF

CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,

ENHANCEMENTS, OR MODIFICATIONS.



                                                PT_COPYRIGHT_VERSION 2

                                                COPYRIGHTENDKEY

*/



package org.geon.test;



import org.geon.*;



import ptolemy.actor.lib.Source;

import ptolemy.data.ArrayToken;

import ptolemy.data.IntToken;

import ptolemy.data.ObjectToken;

import ptolemy.data.StringToken;

import ptolemy.data.Token;

import ptolemy.data.type.ArrayType;

import ptolemy.data.type.BaseType;

import ptolemy.data.expr.FileParameter;

import ptolemy.data.expr.Parameter;

import ptolemy.data.expr.Variable;

import ptolemy.kernel.CompositeEntity;

import ptolemy.kernel.util.*;



import java.io.*;

import java.io.BufferedReader;

import java.io.IOException;

import java.net.URL;

import java.util.Vector;



import nl.skybound.awt.DoublePolygon;



//////////////////////////////////////////////////////////////////////////

//// SVGToPolygon

/**

This actor converts an SVG file into polygon objects.



@author Efrat Jaeger

@version $Id: SVGToPolygon1.java,v 1.2 2004/07/01 23:32:50 higgins Exp $

@since Ptolemy II 3.0.2

*/



public class SVGToPolygon1 extends Source {



  /** Construct an actor with the given container and name.

   *  @param container The container.

   *  @param name The name of this actor.

   *  @exception IllegalActionException If the actor cannot be contained

   *   by the proposed container.

   *  @exception NameDuplicationException If the container already has an

   *   actor with this name.

   */

  public SVGToPolygon1(CompositeEntity container, String name)

          throws IllegalActionException, NameDuplicationException {

      super(container, name);



      fileOrURL = new FileParameter(this, "fileOrURL");



      output.setTypeEquals(BaseType.GENERAL);

      trigger.setTypeEquals(BaseType.STRING);



      _attachText("_iconDescription", "<svg>\n"

              + "<rect x=\"-25\" y=\"-20\" "

              + "width=\"50\" height=\"40\" "

              + "style=\"fill:white\"/>\n"

              + "<polygon points=\"-15,-10 -12,-10 -8,-14 -1,-14 3,-10"

              + " 15,-10 15,10, -15,10\" "

              + "style=\"fill:red\"/>\n"

              + "</svg>\n");

  }



  ///////////////////////////////////////////////////////////////////

  ////                     ports and parameters                  ////



  /** The file name or URL from which to read.  This is a string with

   *  any form accepted by FileParameter.

   *  @see FileParameter

   */

  public FileParameter fileOrURL;



  /** Output the SVG file into a Polygons and their region names.

   *  @exception IllegalActionException If there's no director.

   */

  public void fire() throws IllegalActionException {

    if (trigger.hasToken(0)) {

      String fileName = ((StringToken)trigger.get(0)).stringValue();

//      System.out.println("strFile" + fileName);

      int lineEndInd = fileName.indexOf("\n");

//      System.out.println("CR index: " + lineEndInd);

      if (lineEndInd != -1) { // The string contains a CR.

        fileName = fileName.substring(0, lineEndInd);

      }

      input = new File(fileName.trim());

//      System.out.println("file path " + input.getPath()+"  "+input.exists());

    } else {

      input = fileOrURL.asFile();

    }



    GeonXMLUtil parseXML = new GeonXMLUtil();



    Vector polyPoints = parseXML.getAttrValue(input, "polygon", "points");

    Vector regionsStr = parseXML.getElementsById(input, "polygon", "onmouseover");



    PolygonUtil polygonRegions[] = new PolygonUtil[polyPoints.size()];

    int beginInd, endInd;

    for (int i = 0; i < polyPoints.size(); i++) {



      // extract the polygon points.

      String points = ( (String) polyPoints.get(i));

      int iter = 0, ind = -1, comma = -1;

      polygonRegions[i] = new PolygonUtil();

      polygonRegions[i].Poly = new DoublePolygon();

      while (iter < points.length()) {

        ind = points.indexOf(' ', iter);

        String point = points.substring(iter, ind);

        iter = ind + 1;

        comma = point.indexOf(',');

        double Px = Double.parseDouble(point.substring(0, comma));

        double Py = Double.parseDouble(point.substring(comma + 1));

        System.out.println("x = " + Px + ": y = " + Py);

        polygonRegions[i].Poly.addPoint(Px, Py);

        if (ind == points.lastIndexOf(' ')) {

          point = points.substring(ind + 1);

          comma = point.indexOf(',');

          Px = Double.parseDouble(point.substring(0, comma));

          Py = Double.parseDouble(point.substring(comma + 1));

//          System.out.println("x = " + Px + ": y = " + Py);

          polygonRegions[i].Poly.addPoint(Px, Py);

          iter = points.length();

        }

      }



      // extract the region.

      String reg = ( (String) regionsStr.get(i));

/*      beginInd = reg.indexOf('(');

      endInd = reg.indexOf(')');

      reg = reg.substring(beginInd + 2, endInd - 1);

*/      polygonRegions[i].Region = reg;

    }

    output.broadcast(new ObjectToken(polygonRegions));

  }





  /** Post fire the actor. Return false to indicate that the

  * process has finished. If it returns true, the process will

  * continue indefinitely.

  */

  public boolean postfire() {

      return true;

  }



  ///////////////////////////////////////////////////////////////////

  ////                         private members                   ////



  /** Previous value of fileOrURL parameter. */

  private String _previousFileOrURL;

  private File input;



}

