/* Bundle a sequence of N by M input tokens into a matrix.

 Copyright (c) 1998-2003 The Regents of the University of California.
 All rights reserved.
 Permission is hereby granted, without written agreement and without
 license or royalty fees, to use, copy, modify, and distribute this
 software and its documentation for any purpose, provided that the above
 copyright notice and the following two paragraphs appear in all copies
 of this software.

 IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
 THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 SUCH DAMAGE.

 THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
 CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
 ENHANCEMENTS, OR MODIFICATIONS.

                                        PT_COPYRIGHT_VERSION_2
                                        COPYRIGHTENDKEY

@ProposedRating Yellow (eal@eecs.berkeley.edu)
@AcceptedRating Red (neuendor@eecs.berkeley.edu)
*/

package org.sdm.spa;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridLayout;
import java.util.Vector;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;

import ptolemy.actor.TypedAtomicActor;
import ptolemy.actor.TypedIOPort;
import ptolemy.data.StringToken;
import ptolemy.data.type.BaseType;
import ptolemy.kernel.CompositeEntity;
import ptolemy.kernel.util.Attribute;
import ptolemy.kernel.util.IllegalActionException;
import ptolemy.kernel.util.NameDuplicationException;

//////////////////////////////////////////////////////////////////////////
//// SequenceToMatrix
/**
This actor bundles a specified number of input tokens into a matrix.
On each firing, it reads <i>rows</i> times <i>columns</i> input tokens
and writes one output matrix token with the specified number of rows
and columns.
This actor is polymorphic. It can accept inputs of any scalar type
that has a corresponding matrix type.
<p>

@author Edward Lee
@version $Id: TransfacDisplay.java,v 1.2 2004/05/05 22:26:40 altintas Exp $
@since Ptolemy II 0.4
*/

public class TransfacDisplay extends TypedAtomicActor {
	TfDisplay frame = null;

	/** Construct an actor with the given container and name.
	 *  @param container The container.
	 *  @param name The name of this actor.
	 *  @exception IllegalActionException If the actor cannot be contained
	 *   by the proposed container.
	 *  @exception NameDuplicationException If the container already has an
	 *   actor with this name.
	 */
	public TransfacDisplay(CompositeEntity container, String name)
		throws NameDuplicationException, IllegalActionException {
		super(container, name);

		// default tokenConsumptionRate is 1.
		input = new TypedIOPort(this, "GI", true, false);
		input.setTypeEquals(BaseType.STRING);

		context = new TypedIOPort(this, "Context", true, false);
		context.setTypeEquals(BaseType.STRING);

		// Set the icon.
		_attachText(
			"_iconDescription",
			"<svg>\n"
				+ "<polygon points=\"-15,-15 15,15 15,-15 -15,15\" "
				+ "style=\"fill:white\"/>\n"
				+ "</svg>\n");
	}

	// -- Part of the Actor

	public TypedIOPort input;
	public TypedIOPort context;
	///////////////////////////////////////////////////////////////////
	////                         public methods                    ////

	/** If the argument is <i>rows</i> or <i>columns</i>, then
	 *  set the consumption rate of the input port, and invalidate
	 *  the schedule of the director.
	 *  @param attribute The attribute that has changed.
	 *  @exception IllegalActionException If the parameters are out of range.
	 */
	public void attributeChanged(Attribute attribute)
		throws IllegalActionException {
	}

	/** Consume the inputs and produce the output matrix.
	 *  @exception IllegalActionException If not enough tokens are available.
	 */
	public void fire() throws IllegalActionException {
		super.fire();

		String in = ((StringToken) (input.get(0))).stringValue();
		//Get the context of the result
		String ctx = ((StringToken) (context.get(0))).stringValue();
		_debug("Process:" + in);

		in = in.replace('\\', 'a');
		in = in.replaceAll("a\"", "\"");
		System.out.println(in);
		String temp;
		XMLUtil xu = new XMLUtil();
		Vector mn, pom, cs, ms, seq, ln, tfm, lnt, freq;

		tfm = xu.getItems(in, "/ResultDoc/output/tf_matrix");

		mn = xu.getItems(in, "/ResultDoc/output/tf_matrix/matrixName");

		ln = xu.getItems(in, "/ResultDoc/output/tf_matrix/link");

		pom = xu.getItems(in, "/ResultDoc/output/tf_matrix/positionOfMatrix");

		cs = xu.getItems(in, "/ResultDoc/output/tf_matrix/coreSimilarity");
		seq = xu.getItems(in, "/ResultDoc/output/tf_matrix/sequence");
		ms = xu.getItems(in, "/ResultDoc/output/tf_matrix/matrixSimilarity");

		freq = processTfbs(mn);

		lnt = new Vector();

		//Modify link vector to add information:
		for (int i = 0; i < ln.size(); i++) {
			lnt.addElement("http://transfac.gbf.de" + (String) ln.elementAt(i));
		}

		DefaultTableModel dtm = new DefaultTableModel();

		// Create columns names
		/*String columnNames[] = { "matrixName", "positionOfMatrix", "coreSimilarity", "matrixSimilarity", "sequence" };
		
		// Create some data
		String dataValues[][] =
		{
			  { "12", "234", "67", "00", "11" },
			  { "-123", "43", "853", "00", "11"},
			  { "93", "89.2", "109", "00", "11" },
			  { "279", "9033", "3092" , "00", "11"}
		};*/

		// Create a new table instance
		dtm.addColumn("matrixName", mn);
		dtm.addColumn("link", lnt);
		dtm.addColumn("positionOfMatrix", pom);
		dtm.addColumn("coreSimilarity", cs);
		dtm.addColumn("matrixSimilarity", ms);
		dtm.addColumn("sequence", seq);
		dtm.addColumn("frequencies", freq);

		if (frame == null)
			frame = new TfDisplay();
		frame.addResult(ctx, dtm);
		frame.show();

	}

	class TfDisplay extends JFrame {
		GridLayout gl;
		JPanel outpanel;

		public void addResult(String ctx, DefaultTableModel dtm) {
			JPanel cellpanel;
			JTable table;
			JScrollPane scrollPane;
			
			//add a new row to the grid
			gl.setRows(gl.getRows() + 1);

			// Create a panel to hold all other components
			cellpanel = new JPanel();
			cellpanel.setLayout(new BorderLayout());
			table = new JTable(dtm);
			table.setCellSelectionEnabled(true);
			table.setAutoResizeMode(1);
			table.setColumnSelectionAllowed(true);
			table.setRowSelectionAllowed(true);
			JScrollPane cellsp = new JScrollPane(table);
			cellpanel.setLayout(new BorderLayout());
			cellpanel.add(new JLabel("Context:" + ctx), BorderLayout.NORTH);
			cellpanel.add(cellsp, BorderLayout.CENTER);
			outpanel.add(cellpanel);
			
		}

		TfDisplay() {
			setTitle("Transfac Result Display");
			setSize(400, 300);
			setBackground(Color.gray);
			outpanel = new JPanel();
			gl = new GridLayout();
			gl.setColumns(1);
			gl.setRows(0);
			outpanel.setLayout(gl);
			JScrollPane outsp = new JScrollPane(outpanel);
			getContentPane().add(outsp);

		}
	}

	private Vector processTfbs(Vector mn) {
		String str, str1;
		double tfCount;
		double mnCount = mn.size();
		double frequency;
		Vector uniqueTF = new Vector();
		Vector tfFrequency = new Vector();
		Vector frequencies = new Vector();
		for (int i = 0; i < mn.size(); i++) {
			int index = ((String) mn.elementAt(i)).indexOf("_");
			str = ((String) mn.elementAt(i)).substring(0, index);
			if (uniqueTF.contains(str)) {
			} else {
				uniqueTF.addElement(str);
			}
		}
		//System.out.println("The unique tfbs are" + uniqueTF);

		for (int i = 0; i < uniqueTF.size(); i++) {
			tfCount = 0;
			for (int j = 0; j < mn.size(); j++) {
				if (((String) mn.elementAt(j))
					.startsWith(((String) uniqueTF.elementAt(i)) + "_")) {
					tfCount++;
				}
			}
			//System.out.println("Number of tfbs found with " + uniqueTF.elementAt(i) + "in them is " + tfCount);
			frequency = (tfCount / mnCount) * 100;
			Double d = new Double(frequency);
			tfFrequency.add(i, d);
			//System.out.println("Frequency of tfbs " + uniqueTF.elementAt(i) + "is " + frequency + "%" );

		}
		//System.out.println("The Frequency vector is  " + tfFrequency);

		for (int i = 0; i < mn.size(); i++) {
			int index = ((String) mn.elementAt(i)).indexOf("_");
			str = ((String) mn.elementAt(i)).substring(0, index);
			int freqIndex = uniqueTF.indexOf(str);
			frequencies.addElement(tfFrequency.elementAt(freqIndex) + "%");
		}

		return frequencies;
	}

}
